/*
 * Copyright 2013 Canonical Ltd.
 *
 * This file is part of powerd.
 *
 * powerd is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; version 3.
 *
 * powerd is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <errno.h>
#include <glib.h>
#include <libupower-glib/upower.h>

#include "powerd-internal.h"
#include "log.h"

#define THERMAL_SHUTDOWN_TEMP 68.0f /* xxx */

static UpClient *up_client;

static void up_device_changed_cb(UpClient *client, UpDevice *device,
                                 gpointer unused)
{
    gboolean on_battery;
    GPtrArray *devices;
    int i;
    gboolean low_batt_shutdown = TRUE;
    gboolean thermal_shutdown = FALSE;

    g_object_get(up_client,
                 "on_battery", &on_battery,
                 NULL);
    if (!on_battery)
        low_batt_shutdown = FALSE;
    devices = up_client_get_devices(up_client);
    for (i = 0; i < devices->len; i++) {
        UpDevice *device = g_ptr_array_index(devices, i);
        UpDeviceKind kind;
        gboolean is_present;
        g_object_get(device,
                     "kind", &kind,
                     "is-present", &is_present,
                     NULL);
        if (kind == UP_DEVICE_KIND_BATTERY && is_present) {
            double percentage, temp;
            g_object_get(device,
                         "percentage", &percentage,
                         "temperature", &temp,
                         NULL);
            /*
             * Android shuts down when percentage reaches 0. We
             * use 1% to leave a little more headroom.
             */
            if (percentage > 1.0)
                low_batt_shutdown = FALSE;
            if (temp >= THERMAL_SHUTDOWN_TEMP) {
                powerd_warn("Critical battery temperature %f\n", temp);
                thermal_shutdown = TRUE;
                break;
            }
        }
    }
    if (low_batt_shutdown || thermal_shutdown) {
        powerd_warn("Initiating emergency shutdown");
        powerd_shutdown();
    }
}

int powerd_ps_init(void)
{
    gboolean ret;
    up_client = up_client_new();
    if (!up_client) {
        powerd_warn("Could not allocate upower client");
        return -ENODEV;
    }
    ret = up_client_enumerate_devices_sync(up_client, NULL, NULL);
    if (!ret)
        powerd_warn("Could not enumerate upower devices");
    g_signal_connect(up_client, "device-changed",
                     G_CALLBACK(up_device_changed_cb), NULL);
    return 0;
}

void powerd_ps_deinit(void)
{
    if (up_client)
        g_object_unref(up_client);
}
